package com.xkrs.service.impl;

import com.xkrs.dao.CraftItemDao;
import com.xkrs.dao.QcItemDao;
import com.xkrs.dao.QcSpecDao;
import com.xkrs.encapsulation.PromptMessageEnum;
import com.xkrs.model.bean.ReadSpecHeadBean;
import com.xkrs.model.bean.XSSFCellDataBean;
import com.xkrs.model.entity.CraftItemEntity;
import com.xkrs.model.entity.QcItemEntity;
import com.xkrs.model.entity.QcSpecEntity;
import com.xkrs.model.qo.ExportSpecExcelQo;
import com.xkrs.model.qo.QcSpecQoDelete;
import com.xkrs.model.qo.QcSpecQoInsert;
import com.xkrs.model.qo.QcSpecQoUpdate;
import com.xkrs.service.QcSpecService;
import com.xkrs.util.LocalDateUtils;
import com.xkrs.util.LocalDoubleUtils;
import com.xkrs.util.LocalStringUtils;
import org.apache.http.util.TextUtils;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFDateUtil;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.awt.*;
import java.io.File;
import java.io.FileOutputStream;
import java.util.List;
import java.util.*;

import static com.xkrs.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@Service
public class QcSpecServiceImpl implements QcSpecService {

    private Locale locale = LocaleContextHolder.getLocale();

    @Resource
    private CraftItemDao craftItemDao;

    @Resource
    private QcItemDao qcItemDao;

    @Resource
    private QcSpecDao qcSpecDao;

    /**
     * 添加检验规格
     */
    @Override
    public String insertQcSpec(QcSpecQoInsert insertQo) {

        String varietyNo = insertQo.getVarietyNo();
        String craftItemNo = insertQo.getCraftItemNo();
        String qcItemNo = insertQo.getQcItemNo();
        String max = insertQo.getMax();
        String mean = insertQo.getMean();
        String min = insertQo.getMin();
        String unit = insertQo.getUnit();
        String method = insertQo.getMethod();
        String standard = insertQo.getStandard();
        String remark = insertQo.getRemark();

        Optional<CraftItemEntity> craftItemByNo = craftItemDao.findByCraftItemNo(craftItemNo);
        if (craftItemByNo.isEmpty()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "不存在该工艺项目，请先添加工艺项目！", locale);
        }
        Optional<QcItemEntity> qcItemByNo = qcItemDao.findByQcItemNo(qcItemNo);
        if (qcItemByNo.isEmpty()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "不存在该检验项目，请先添加检验项目！", locale);
        }
        Optional<QcSpecEntity> existsQcSpecOptional = qcSpecDao.findExistsQcSpec(varietyNo, craftItemNo, qcItemNo);
        if (existsQcSpecOptional.isPresent()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_EXIT, "该检验规格已存在，请勿重复添加！", locale);
        }
        QcSpecEntity entity = new QcSpecEntity();
        entity.setCreateTime(LocalDateUtils.getCurrentSecond());
        entity.setUpdateTime("");
        entity.setVarietyNo(LocalStringUtils.formatEmptyValue(varietyNo));
        entity.setCraftItemNo(LocalStringUtils.formatEmptyValue(craftItemNo));
        entity.setQcItemNo(LocalStringUtils.formatEmptyValue(qcItemNo));
        entity.setMax(LocalStringUtils.formatEmptyValue(max));
        entity.setMean(LocalStringUtils.formatEmptyValue(mean));
        entity.setMin(LocalStringUtils.formatEmptyValue(min));
        entity.setUnit(LocalStringUtils.formatEmptyValue(unit));
        entity.setMethod(LocalStringUtils.formatEmptyValue(method));
        entity.setStandard(LocalStringUtils.formatEmptyValue(standard));
        entity.setRemark(LocalStringUtils.formatEmptyValue(remark));
        qcSpecDao.save(entity);

        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "添加成功！", locale);
    }

    /**
     * 删除检验规格
     */
    @Transactional(rollbackOn = Exception.class)
    @Override
    public String deleteQcSpec(QcSpecQoDelete deleteQo) {
        Integer id = deleteQo.getId();
        Optional<QcSpecEntity> entityById = qcSpecDao.findById(id);
        if (entityById.isEmpty()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "不存在该检验规格，请先添加检验规格！", locale);
        }
        qcSpecDao.deleteById(id);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "删除成功！", locale);
    }

    /**
     * 更新检验规格
     */
    @Override
    public String updateQcSpec(QcSpecQoUpdate updateQo) {

        Integer id = updateQo.getId();
        String max = updateQo.getMax();
        String mean = updateQo.getMean();
        String min = updateQo.getMin();
        String unit = updateQo.getUnit();
        String method = updateQo.getMethod();
        String standard = updateQo.getStandard();
        String remark = updateQo.getRemark();

        Optional<QcSpecEntity> entityById = qcSpecDao.findById(id);
        if (entityById.isEmpty()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "不存在该检验规格，请先添加检验规格！", locale);
        }

        qcSpecDao.updateQcSpecById(id, LocalDateUtils.getCurrentSecond(), LocalStringUtils.formatEmptyValue(max), LocalStringUtils.formatEmptyValue(mean), LocalStringUtils.formatEmptyValue(min), LocalStringUtils.formatEmptyValue(unit), LocalStringUtils.formatEmptyValue(method), LocalStringUtils.formatEmptyValue(standard), LocalStringUtils.formatEmptyValue(remark));

        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, "修改成功！", locale);
    }

    /**
     * 查询检验规格
     */
    @Override
    public String queryQcSpec(String varietyNo, String craftItemNo, String qcItemNo) {
        String finalVarietyNo = LocalStringUtils.formatEmptyValue(varietyNo).trim();
        String finalQcItemNo = LocalStringUtils.formatEmptyValue(qcItemNo).trim();
        List<Map<String, Object>> resultList;
        if (TextUtils.isEmpty(craftItemNo)) {
            resultList = qcSpecDao.queryQcSpec(finalVarietyNo, finalQcItemNo);
        } else {
            resultList = qcSpecDao.queryQcSpec(finalVarietyNo, craftItemNo.trim(), finalQcItemNo);
        }
        if (resultList == null || resultList.isEmpty()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "暂时没有该检验规格的信息！", locale);
        }
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, resultList, locale);
    }

    /**
     * 导入检验规格
     */
    @Override
    public String importSpecExcel(MultipartFile specExcel) {
        try {
            //1、读取检验规格数据表文件
            XSSFWorkbook workbook = new XSSFWorkbook(specExcel.getInputStream());
            //2、获取工作表对象
            XSSFSheet sheet = workbook.getSheetAt(0);
            //3、获取行的迭代器
            Iterator<Row> iterator = sheet.iterator();
            long rowNum = 0;
            long colNum = -1;
            //4、表格数据容器
            List<List<String>> sheetDataList = new ArrayList<>();
            while (iterator.hasNext()) {
                Row row = iterator.next();
                if (rowNum == 0) {
                    ReadSpecHeadBean readResult = readSpecHead(row);
                    if (readResult.isSuccess()) {
                        colNum = readResult.getHeadCount();
                        rowNum++;
                        continue;
                    }
                    return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, readResult.getErrorMessage(), locale);
                }
                List<String> rowDataList = new ArrayList<>();
                for (int i = 0; i < colNum; i++) {
                    String value = getValue(row.getCell(i));
                    if (i == 0) {//机种号数据直接添加
                        if (TextUtils.isEmpty(value)) {
                            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "第" + rowNum + "行机种号为空，请先编辑机种号！", locale);
                        }
                        rowDataList.add(value);
                    } else if (i == 1) {//工艺项目数据要添加对应的编号
                        Optional<CraftItemEntity> craftItemOptional = craftItemDao.findByCraftItemName(value);
                        if (craftItemOptional.isEmpty()) {
                            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "第" + rowNum + "行工艺项目（" + value + "）不存在，请先添加工艺项目！", locale);
                        }
                        rowDataList.add(craftItemOptional.get().getCraftItemNo());
                    } else if (i == 2) {//检验项目数据要添加对应的编号
                        Optional<QcItemEntity> qcItemOptional = qcItemDao.findByQcItemName(value);
                        if (qcItemOptional.isEmpty()) {
                            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "第" + rowNum + "行检验项目（" + value + "）不存在，请先添加检验项目！", locale);
                        }
                        rowDataList.add(qcItemOptional.get().getQcItemNo());
                    } else {//其他数据直接添加
                        rowDataList.add(value);
                    }
                }
                sheetDataList.add(rowDataList);
                rowNum++;
            }
            //5、表格数据自重复检查
            for (int i = 0; i < sheetDataList.size(); i++) {
                List<String> rowDataList = sheetDataList.get(i);
                List<Integer> appearIndexList = checkAppearIndex(sheetDataList, rowDataList.get(0), rowDataList.get(1), rowDataList.get(2));
                if (appearIndexList.size() == 0) {
                    return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "系统错误！", locale);
                }
                if (appearIndexList.size() > 1) {
                    StringBuilder response = new StringBuilder();
                    response.append("检验规格表中：机种号、工艺项目、检验项目三项存在全部相同项，行数：");
                    for (Integer appearIndex : appearIndexList) {
                        response.append("第").append(appearIndex + 2).append("行、");
                    }
                    return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, response.toString(), locale);
                }
            }
            //6、表格的数据已经全部读取完成，并且自检通过，接下来遍历入库即可
            int insertCount = 0;
            int updateCount = 0;
            for (List<String> rowDataList : sheetDataList) {
                String varietyNo = rowDataList.get(0);
                String craftItemNo = rowDataList.get(1);
                String qcItemNo = rowDataList.get(2);
                String max = rowDataList.get(3);
                String min = rowDataList.get(4);
                String method = rowDataList.get(5);
                Optional<QcSpecEntity> existsQcSpecOptional = qcSpecDao.findExistsQcSpec(varietyNo, craftItemNo, qcItemNo);
                if (existsQcSpecOptional.isPresent()) {
                    qcSpecDao.updateQcSpecByVCQ(LocalStringUtils.formatEmptyValue(varietyNo), LocalStringUtils.formatEmptyValue(craftItemNo), LocalStringUtils.formatEmptyValue(qcItemNo), LocalDateUtils.getCurrentSecond(), LocalStringUtils.formatEmptyValue(max), "", LocalStringUtils.formatEmptyValue(min), "", LocalStringUtils.formatEmptyValue(method), "", "");
                    updateCount++;
                } else {
                    QcSpecEntity entity = new QcSpecEntity();
                    entity.setCreateTime(LocalDateUtils.getCurrentSecond());
                    entity.setUpdateTime("");
                    entity.setVarietyNo(LocalStringUtils.formatEmptyValue(varietyNo));
                    entity.setCraftItemNo(LocalStringUtils.formatEmptyValue(craftItemNo));
                    entity.setQcItemNo(LocalStringUtils.formatEmptyValue(qcItemNo));
                    entity.setMax(LocalStringUtils.formatEmptyValue(max));
                    entity.setMean("");
                    entity.setMin(LocalStringUtils.formatEmptyValue(min));
                    entity.setUnit("");
                    entity.setMethod(LocalStringUtils.formatEmptyValue(method));
                    entity.setStandard("");
                    entity.setRemark("");
                    qcSpecDao.save(entity);
                    insertCount++;
                }
            }
            StringBuilder response = new StringBuilder();
            response.append("导入成功");
            if (insertCount > 0) {
                response.append("，新增").append(insertCount).append("条检验规格");
            }
            if (updateCount > 0) {
                response.append("，更新").append(updateCount).append("条检验规格");
            }
            response.append("。");
            return outputEncapsulationObject(PromptMessageEnum.SUCCESS, response.toString(), locale);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "请上传正确的Microsoft Excel 2007/2010 Workbook(*.xlsx)文件！", locale);
    }

    private List<Integer> checkAppearIndex(List<List<String>> sheetDataList, String varietyNo, String craftItemNo, String qcItemNo) {
        List<Integer> appearIndexList = new ArrayList<>();
        for (int i = 0; i < sheetDataList.size(); i++) {
            List<String> rowDataList = sheetDataList.get(i);
            if (rowDataList.get(0).equals(varietyNo) && rowDataList.get(1).equals(craftItemNo) && rowDataList.get(2).equals(qcItemNo)) {
                appearIndexList.add(i);
            }
        }
        return appearIndexList;
    }

    List<String> QC_SPEC_HEAD = Arrays.asList("机种号", "工艺项目", "检验项目", "最大值", "最小值", "检验方法", "最大值(管理值)", "最小值(管理值)");

    /**
     * 读取表头数据
     */
    private ReadSpecHeadBean readSpecHead(Row headRow) {
        ReadSpecHeadBean readResult = new ReadSpecHeadBean();
        for (int i = 0; i < 6; i++) {
            if (!QC_SPEC_HEAD.get(i).equals(getValue(headRow.getCell(i)))) {
                readResult.setSuccess(false);
                readResult.setHeadCount(0);
                readResult.setErrorMessage("表头第" + (i + 1) + "列应该是：" + QC_SPEC_HEAD.get(i));
                return readResult;
            }
        }
        readResult.setSuccess(true);
        readResult.setHeadCount(6);
        readResult.setErrorMessage("");
        return readResult;
    }

    /**
     * 获取单元格内的数据，并进行格式转换
     */
    private String getValue(Cell cell) {
        if (cell == null) {
            return "";
        }
        if (Cell.CELL_TYPE_STRING == cell.getCellType()) {
            return cell.getStringCellValue();
        }
        if (Cell.CELL_TYPE_BOOLEAN == cell.getCellType()) {
            return String.valueOf(cell.getBooleanCellValue());
        }
        if (Cell.CELL_TYPE_NUMERIC == cell.getCellType()) {//数值和日期均是此类型,需进一步判断
            if (HSSFDateUtil.isCellDateFormatted(cell)) {
                //是日期类型
                return String.valueOf(cell.getDateCellValue().getTime());
            } else {
                //是数值类型
                return String.valueOf(cell.getNumericCellValue());
            }
        }
        return "";

    }

    /**
     * 导出检验规格
     */
    @Override
    public String exportSpecExcel(ExportSpecExcelQo exportSpecExcelQo) throws Exception {

        List<QcSpecEntity> qcSpecList = qcSpecDao.findAll(Sort.by(Sort.Direction.ASC, "createTime"));
        if (qcSpecList.isEmpty()) {
            return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "无检验规格数据可导出，请先添加检验规格！", locale);
        }

        List<List<String>> dataListList = new ArrayList<>();
        dataListList.add(QC_SPEC_HEAD);
        for (QcSpecEntity qcSpecEntity : qcSpecList) {
            List<String> dataList = new ArrayList<>();
            dataList.add(qcSpecEntity.getVarietyNo());
            Optional<CraftItemEntity> craftItemOptional = craftItemDao.findByCraftItemNo(qcSpecEntity.getCraftItemNo());
            if (craftItemOptional.isEmpty()) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "检验规格数据导出失败（code=01）！", locale);
            }
            dataList.add(craftItemOptional.get().getCraftItemName());
            Optional<QcItemEntity> qcItemOptional = qcItemDao.findByQcItemNo(qcSpecEntity.getQcItemNo());
            if (qcItemOptional.isEmpty()) {
                return outputEncapsulationObject(PromptMessageEnum.DATA_NONE, "检验规格数据导出失败（code=02）！", locale);
            }
            dataList.add(qcItemOptional.get().getQcItemName());
            dataList.add(qcSpecEntity.getMax());
            dataList.add(qcSpecEntity.getMin());
            dataList.add(qcSpecEntity.getMethod());
            dataListList.add(dataList);
        }

        //创建XSSFWorkbook对象(excel的文档对象)
        XSSFWorkbook workbook = new XSSFWorkbook();
        //建立新的sheet对象（excel的表单）
        XSSFSheet sheet = workbook.createSheet("检验规格表");
        for (int i = 0; i < dataListList.get(0).size(); i++) {
            sheet.setColumnWidth(i, 24 * 256);
        }

        XSSFCellStyle headCellStyle = workbook.createCellStyle();
        headCellStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        headCellStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        XSSFFont headFont = workbook.createFont();
        headFont.setColor(HSSFColor.BLACK.index);
        headCellStyle.setFont(headFont);
        headCellStyle.setFillForegroundColor(new XSSFColor(new Color(156, 195, 230)));//设置背景色
        headCellStyle.setFillPattern(CellStyle.SOLID_FOREGROUND);//填充模式

        XSSFCellStyle normalCellStyle = workbook.createCellStyle();
        normalCellStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        normalCellStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        XSSFFont normalFont = workbook.createFont();
        normalFont.setColor(HSSFColor.BLACK.index);
        normalCellStyle.setFont(normalFont);

        int startRowNum = 0;
        for (int i = 0; i < dataListList.size(); i++) {
            List<String> dataList = dataListList.get(i);
            if (dataList != null && dataList.size() > 0) {
                XSSFRow row = sheet.createRow(i + startRowNum);
                for (int j = 0; j < dataList.size(); j++) {
                    XSSFCell cell = row.createCell(j);
                    String sourceValue = dataList.get(j);
                    if (i == 0) {
                        cell.setCellValue(sourceValue);
                        cell.setCellStyle(headCellStyle);
                    } else if (j <= 2) {
                        cell.setCellValue(sourceValue);
                        cell.setCellStyle(normalCellStyle);
                    } else {
                        XSSFCellDataBean cellDataBean = new XSSFCellDataBean(sourceValue);
                        if (XSSFCellDataBean.VALUE_TYPE_STRING == cellDataBean.getValueType()) {
                            cell.setCellValue(cellDataBean.getStringValue());
                        } else if (XSSFCellDataBean.VALUE_TYPE_DOUBLE == cellDataBean.getValueType()) {
                            cell.setCellValue(LocalDoubleUtils.keepThreeDecimal(cellDataBean.getDoubleValue()));
                        }
                        cell.setCellStyle(normalCellStyle);
                    }
                }
            }
        }
        //输出Excel文件
        File dirFile = new File("/home/sxy/server/industrial_measurement/excel/");
//        File dirFile = new File("/Users/liuchengqian/Desktop/DaJiang/");
        if (!dirFile.exists()) {
            dirFile.mkdirs();
        }
        String excelFileName = System.currentTimeMillis() + ".xlsx";
        FileOutputStream output = new FileOutputStream(new File(dirFile, excelFileName));
        workbook.write(output);
        output.flush();

        String host = exportSpecExcelQo.getHost();
        String finalHost = host.endsWith("/") ? host : host + "/";
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, finalHost + "excel/" + excelFileName, locale);
    }

    private boolean formatData(String data) {
        return false;
    }


    /**
     * 导出空白检验规格
     */
    @Override
    public String exportEmptySpecExcel(ExportSpecExcelQo exportSpecExcelQo) throws Exception {

        List<List<String>> dataListList = new ArrayList<>();
        dataListList.add(QC_SPEC_HEAD);

        XSSFWorkbook workbook = new XSSFWorkbook();
        //建立新的sheet对象（excel的表单）
        XSSFSheet sheet = workbook.createSheet("检验规格表");
        for (int i = 0; i < dataListList.get(0).size(); i++) {
            sheet.setColumnWidth(i, 24 * 256);
        }

        XSSFCellStyle headCellStyle = workbook.createCellStyle();
        headCellStyle.setAlignment(HSSFCellStyle.ALIGN_CENTER);
        headCellStyle.setVerticalAlignment(HSSFCellStyle.VERTICAL_CENTER);
        XSSFFont headFont = workbook.createFont();
        headFont.setColor(HSSFColor.BLACK.index);
        headCellStyle.setFont(headFont);
        headCellStyle.setFillForegroundColor(new XSSFColor(new Color(156, 195, 230)));//设置背景色
        headCellStyle.setFillPattern(CellStyle.SOLID_FOREGROUND);//填充模式

        int startRowNum = 0;
        List<String> dataList = dataListList.get(0);
        if (dataList != null && dataList.size() > 0) {
            XSSFRow row = sheet.createRow(startRowNum);
            for (int j = 0; j < dataList.size(); j++) {
                XSSFCell cell = row.createCell(j);
                cell.setCellStyle(headCellStyle);
                String data = dataList.get(j);
                cell.setCellValue(data);
            }
        }

        //输出Excel文件
        File dirFile = new File("/home/sxy/server/industrial_measurement/excel/");
//        File dirFile = new File("/Users/liuchengqian/Desktop/DaJiang/");
        if (!dirFile.exists()) {
            dirFile.mkdirs();
        }
        String excelFileName = System.currentTimeMillis() + ".xlsx";
        FileOutputStream output = new FileOutputStream(new File(dirFile, excelFileName));
        workbook.write(output);
        output.flush();

        String host = exportSpecExcelQo.getHost();
        String finalHost = host.endsWith("/") ? host : host + "/";
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, finalHost + "excel/" + excelFileName, locale);
    }
}
